class TheCatTools.GUI.BoubleCircle : Adw.Bin, Gtk.Orientable {
    public int requested_size { get; set; default = 60; }
    public Gtk.Adjustment adjustment { get; set; default = new Gtk.Adjustment (0, -90, 90, 0, 0, 0); }
    public Gtk.Orientation orientation { get; set; }
    public Bubble bubble = new Bubble ();
    public Circle clamps_ok = new Circle ();
    public Gtk.Overlay overlay = new Gtk.Overlay ();

    construct {
        width_request = 60;
        add_css_class ("bubble-circle");
        notify["value"].connect (queue_draw);
        notify["orientation"].connect (orientation_changed);
        orientation_changed ();
        bubble.vexpand = true;
        bubble.hexpand = true;
        overlay.add_overlay (clamps_ok);
        overlay.add_overlay (bubble);
        set_child (overlay);
        bind_property ("orientation", clamps_ok, "orientation", GLib.BindingFlags.SYNC_CREATE);
        bind_property ("orientation", bubble, "orientation", GLib.BindingFlags.SYNC_CREATE);
        clamps_ok.bind_state_to_bubble_pos (bubble);
    }

    public void orientation_changed () {
        var orientation = orientation == Gtk.Orientation.HORIZONTAL;
        width_request = -1;
        width_request = orientation ? -1 : requested_size;
        height_request = orientation ? requested_size : -1;
    }

    public class Circle : DrawingAreaOrientable {
        public Gtk.Adjustment adjustment { get; set; default = new Gtk.Adjustment (0, -90, 90, 0, 0, 0); }
        public Gtk.Adjustment clamps { get; set; default = new Gtk.Adjustment (0, -10, 10, 0, 0, 0); }
        construct {
            add_css_class ("circle");
        }
        public void bind_state_to_bubble_pos (Bubble bubble) {
            bubble.notify["value-x"].connect (() => {
                bool inb = true;
                if (bubble.value_x >= clamps.upper || bubble.value_y >= clamps.upper) {
                    set_state_flags (Gtk.StateFlags.CHECKED, false);
                    inb = false;
                } else if (bubble.value_x <= clamps.lower || bubble.value_y <= clamps.lower) {
                    set_state_flags (Gtk.StateFlags.CHECKED, false);
                    inb = false;
                } else
                    unset_state_flags (Gtk.StateFlags.CHECKED);
            });
        }

        public override void draw_func (Gtk.DrawingArea self, Cairo.Context cr, int width, int height) {
            var range = adjustment.upper - adjustment.lower;
            Utils.set_color_rgba (cr, get_color ());
            var radius = width / range * (clamps.upper - clamps.lower);
            cr.arc (width / 2, height / 2, radius, 0, Math.PI * 2);
            cr.stroke ();
        }
    }
}