/*
 * Copyright 2025 Vasiliy Doylov <nekodevelopper@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */
double rad2deg(double angle) {
    return angle / Math.PI * 180;
}

[GtkTemplate(ui = "/io/gitlab/nekocwd/the-cat-tools/window.ui")]
public class TheCatTools.Window : Adw.ApplicationWindow {
    [GtkChild]
    private unowned GUI.BoubleBar hbar;
    [GtkChild]
    private unowned GUI.BoubleBar vbar;
    [GtkChild]
    private unowned GUI.BoubleCircle circle;
    private SSC.SensorAccelerometer accel;
    private SSC.SensorGyroscope gyro;


    async void start_sensors() {
        try {
            accel = new SSC.SensorAccelerometer.sync(null);
            message("Sensor created");
            if (accel == null)
                message("Sensor is NULL");
            accel.open_sync();
            accel.measurement.connect((x, y, z) => {
                var x_angle = rad2deg(Math.atan2(x, z));
                var y_angle = rad2deg(Math.atan2(y, z));

                hbar.bubble.value_x = (float) x_angle;
                vbar.bubble.value_x = (float) y_angle;
                circle.bubble.value_x = (float) x_angle;
                circle.bubble.value_y = (float) y_angle;
            });
            gyro = new SSC.SensorGyroscope.sync(null);
            message("Sensor created");
            if (accel == null)
                message("Sensor is NULL");
            gyro.open_sync();
            gyro.measurement.connect((x, y, z) => {
                message("%f\t%f\t%f m/s", x, y, z);
            });
            
        } catch (Error err) {
            message("Failed to open Sensor %s", err.message);
        }
    }

    public Window(Gtk.Application app) {
        Object(application: app);
        start_sensors.begin();
    }
}